# Language

## `async` Closures

2021:

Have to use a generic type because impl Trait is not allowed in return
specifications of functions.

```
use core::future::Future;

async fn asynchronous<F>(callback: impl FnOnce(String) -> F)
where F: Future<Output = ()>
{
    callback("Hello, world!".into()).await
}

#[tokio::main]
async fn main() {
    asynchronous(|message| async move {
        println!("{}", message);
    }).await;
}
```

2024:

Notice that the `async` moved to the front, indicating an `async` closure and
the `move` was removed. This is because Futures returned by async blocks are
not allowed to borrow from the closure context, so we had to `move` message
into the async block. Now, the future returned by this function is able to
borrow `message`.

```
async fn asynchronous(callback: impl AsyncFnOnce(String))
{
    callback("Hello, world!".into()).await
}

#[tokio::main]
async fn main() {
    asynchronous(async |message| {
        println!("{}", message);
    }).await;
}
```

In 2021, there were also some weird holes around higher-order `async`
functions:

```
use core::future::Future;
async fn f<Fut>(_: impl for<'a> Fn(&'a u8) -> Fut)
where
    Fut: Future<Output = ()>,
{ todo!() }

async fn f2(_: impl for<'a> AsyncFn(&'a u8))
{ todo!() }

async fn main() {
    async fn g(_: &u8) { todo!() }
    f(g).await;
    //~^ ERROR mismatched types
    //~| ERROR one type is more general than the other

    f2(g).await; // ok!
}
```

## RPIT Lifetime Capture Rules

https://crates.io/crates/fix-hidden-lifetime-bug

The `use` syntax is just language-level support for this trick.

2021:

This code fails to compile:

```
fn foo<'a>(x: &'a()) -> impl Sized {
    x
}
```

https://doc.rust-lang.org/reference/trait-bounds.html#lifetime-bounds

I can't add `+ 'a`, because that indicates that all generic lifetime parameters
of the concrete type _outlive_ `'a`. That's very important. They don't outlive
`'a`, they live _exactly as long as `'a`_. I guess it introduces unsoundness to
change this.

The `use` syntax is for this:

```
fn foo<'a>(x: &'a()) -> impl Sized + use<'a> {
    x
}
```

This is called _Precise Capturing_.

In the 2024 edition, however, `use<>` is not required--the generic lifetime
`'a` is captured automatically. 

That doesn't mean we don't need to understand precise capturing, though,
because this introduces a different footgun if the concrete type does not
actually capture the lifetime:

```
fn foo<'a>(x: &'a()) -> impl Sized {
    1
}

fn captures<'a>(x: &'a()) -> impl Sized + 'static {
    foo(x)
}
```

In this case, the return type of `foo` doesn't need to capture the generic
lifetime `'a` (because it returns `1`), but it does. This prevents it from
being used in the definition of `captures`. To fix this, we can add `+ use<>`
to the definition of `foo`.

https://doc.rust-lang.org/edition-guide/rust-2024/rpit-lifetime-capture.html

## `FromIterator` and `Extend` for tuples

```
use std::collections::{LinkedList, VecDeque};
fn main() {
    let (squares, cubes, tesseracts): (Vec<_>, VecDeque<_>, LinkedList<_>) =
        (0i32..10).map(|i| (i * i, i.pow(3), i.pow(4))).collect();
    println!("{squares:?}");
    println!("{cubes:?}");
    println!("{tesseracts:?}");
}
```

## `if let` Temporary Scope

https://doc.rust-lang.org/edition-guide/rust-2024/temporary-if-let-scope.html

## Tail Expression Temporary Scope

https://doc.rust-lang.org/edition-guide/rust-2024/temporary-tail-expr-scope.html

## Match Ergonomics Reservations

https://doc.rust-lang.org/edition-guide/rust-2024/match-ergonomics.html

## Unsafe `extern` blocks and attributes

Rust 1.82 added the ability to mark `extern` blocks and attributes as `unsafe`.
This edition requires all `extern` blocks to be marked `unsafe`, and some
attributes are not required to be marked `unsafe`.

https://doc.rust-lang.org/edition-guide/rust-2024/unsafe-extern.html
https://doc.rust-lang.org/edition-guide/rust-2024/unsafe-attributes.html

## `unsafe_op_in_unsafe_fn`

This is a lint!

Summary: Everything that requires unsafe should be marked unsafe!

https://doc.rust-lang.org/edition-guide/rust-2024/unsafe-op-in-unsafe-fn.html

## Disallow References to `static mut`

This is a lint!

https://doc.rust-lang.org/edition-guide/rust-2024/static-mut-references.html

Merely taking such a reference in violation of Rust's mutability XOR aliasing
requirement has always been instantaneous undefined behavior, even if the
reference is never read from or written to.

## Never Type Fallback Change

Prevents a simple confusing type inference issue.

https://doc.rust-lang.org/edition-guide/rust-2024/never-type-fallback.html
https://en.wikipedia.org/wiki/Bottom_type

## Macro Fragment Specifiers

`expr` fragment specifier now _also_ matches `const` expressions:

https://doc.rust-lang.org/edition-guide/rust-2024/macro-fragment-specifiers.html

## Missing Macro Fragment Specifiers

More lints!

## `gen` Keyword

This keyword is reserved, which allows us to stabilize `gen` blocks before the
next edition!

https://rust-lang.github.io/rfcs/3513-gen-blocks.html

# Standard Library

* Adds `Future` and `IntoFuture` to the prelude
* Add `IntoIterator` for `Box<[T]>`

## Newly Unsafe Functions

https://doc.rust-lang.org/edition-guide/rust-2024/newly-unsafe-functions.html

`std::env::{set_var, remove_var}`: These aren't thread-safe (Whoops!).

`std::os::unix::process::CommandExt::before_exec`: Is deprecated anyways, in
favor of `pre_exec`, which is marked safe.

# Cargo

https://doc.rust-lang.org/edition-guide/rust-2024/cargo-resolver.html

# Rustdoc

* Rustdoc Combined Tests

# How to Migrate

* https://doc.rust-lang.org/edition-guide/editions/transitioning-an-existing-project-to-a-new-edition.html
